#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  checkBox.py
@Date    :  2021/05/08
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""

from PyQt5.QtWidgets import QCheckBox


class CheckBox(QCheckBox):
    def __init__(self, text: str = "", checked: bool = False):
        super(CheckBox, self).__init__()
        self.setChecked(checked)
        self.setText(text)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  lineEdit.py
@Date    :  2021/05/08
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QLineEdit, QAction


class LineEdit(QLineEdit):
    def __init__(self, placeholderText: str = "", iconUrl: str = ''):
        super(LineEdit, self).__init__()
        self.setPlaceholderText(placeholderText)

        if iconUrl != '':
            action = QAction(self)
            action.setIcon(QIcon(iconUrl))
            self.addAction(action, QLineEdit.LeadingPosition)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  taskItemView.py
@Date    :  2021/9/14
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QWidget, QHBoxLayout, QVBoxLayout

from tidal_gui.control.label import Label
from tidal_gui.control.layout import createHBoxLayout, createVBoxLayout
from tidal_gui.control.listWidget import ListWidget
from tidal_gui.control.pushButton import PushButton
from tidal_gui.style import LabelStyle, ButtonStyle, ListWidgetStyle


class TaskItemView(QWidget):
    def __init__(self):
        super().__init__()
        self.__initView__()
        self.setObjectName('TaskItemView')
        self.setAttribute(Qt.WA_StyledBackground)

    def __initView__(self):
        layout = QVBoxLayout()
        layout.addLayout(self.__initHead__(), Qt.AlignTop)
        layout.addWidget(self.__initList__())

        self.setLayout(layout)

    def __initHead__(self):
        self._btnRetry = PushButton('', ButtonStyle.TaskRetry)
        self._btnCancel = PushButton('', ButtonStyle.TaskCancel)
        self._btnDelete = PushButton('', ButtonStyle.TaskDelete)
        self._btnOpen = PushButton('', ButtonStyle.TaskOpen)
        self._btnExpand = PushButton('', ButtonStyle.TaskExpand)
        self._btnExpand.clicked.connect(self.__expandClick__)
        btnLayout = createHBoxLayout([self._btnRetry, 
                                      self._btnCancel,
                                      #self._btnDelete, 
                                      self._btnOpen,
                                      self._btnExpand])

        self._titleLabel = Label('', LabelStyle.PageTitle)
        self._descLabel = Label()
        self._errLabel = Label()
        self._errLabel.hide()
        labelLayout = createVBoxLayout([self._titleLabel, self._descLabel, self._errLabel])
        labelLayout.insertStretch(0, 1)
        labelLayout.addStretch(1)

        self._picLabel = Label('', LabelStyle.Icon)
        self._picLabel.setMinimumHeight(64)
        headLayout = QHBoxLayout()
        headLayout.addWidget(self._picLabel)
        headLayout.addLayout(labelLayout)
        headLayout.addStretch(1)
        headLayout.addLayout(btnLayout)

        return headLayout

    def __initList__(self):
        self._list = QWidget()
        self._list.setObjectName("DownloadItemsWidget")
        self._listLayout = QVBoxLayout(self._list)
        self._listLayout.setSpacing(0)
        return self._list

    def setLabel(self, title, desc):
        self._titleLabel.setText(title)
        self._descLabel.setText(desc)

    def setErrmsg(self, msg):
        self._errLabel.setText(msg)

    def setPic(self, data):
        pic = QPixmap()
        pic.loadFromData(data)
        self._picLabel.setPixmap(pic.scaled(64, 64))

    def addListItem(self, view):
        self._listLayout.addWidget(view)

    def __expandClick__(self):
        if self._list.isHidden():
            self._list.setVisible(True)
        else:
            self._list.setVisible(False)

    def connectButton(self, name: str, func):
        if name == 'retry':
            self._btnRetry.clicked.connect(func)
        elif name == 'cancel':
            self._btnCancel.clicked.connect(func)
        elif name == 'delete':
            self._btnDelete.clicked.connect(func)
        elif name == 'open':
            self._btnOpen.clicked.connect(func)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  searchModel.py
@Date    :  2021/8/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
import _thread
import threading

import aigpy.stringHelper
from PyQt5.QtCore import pyqtSignal
from aigpy.modelHelper import ModelBase

import tidal_dl
from tidal_dl import Type
from tidal_dl.util import API, getAudioQualityList, getCurAudioQuality, getCurVideoQuality, getVideoQualityList, setCurAudioQuality, setCurVideoQuality
from tidal_gui.view.searchView import SearchView
from tidal_gui.viewModel.viewModel import ViewModel


class SearchModel(ViewModel):
    SIGNAL_ADD_TASKITEM = pyqtSignal(ModelBase)

    def __init__(self):
        super(SearchModel, self).__init__()
        self._lock = threading.Lock()
        self._resultData = None

        self.view = SearchView()
        self.view.setPageIndex(1, 1)
        self.view.setTrackQualityItems(getAudioQualityList(), getCurAudioQuality())
        self.view.setVideoQualityItems(getVideoQualityList(), getCurVideoQuality())
        self.view.connectButton('search', self.__search__)
        self.view.connectButton('prePage', self.__searchPre__)
        self.view.connectButton('nextPage', self.__searchNext__)
        self.view.connectButton('download', self.__download__)
        self.view.connectQualityComboBox('track', self.__changeAudioQuality__)
        self.view.connectQualityComboBox('video', self.__changeVideoQuality__)
        self.view.connectTab(lambda: self.__search__(0))
        self.SIGNAL_REFRESH_VIEW.connect(self.__refresh__)

    def __getSumByResult__(self, stype: Type):
        if stype == Type.Album:
            return self._resultData.albums.totalNumberOfItems
        elif stype == Type.Artist:
            return self._resultData.artists.totalNumberOfItems
        elif stype == Type.Track:
            return self._resultData.tracks.totalNumberOfItems
        elif stype == Type.Video:
            return self._resultData.videos.totalNumberOfItems
        elif stype == Type.Playlist:
            return self._resultData.playlists.totalNumberOfItems
        return 0

    def __refresh__(self, stype: str, data):
        if stype == 'setPageIndex':
            self.view.setPageIndex(data[0], data[1])
        elif stype == 'setTableItems':
            self.view.setTableItems(data[0], data[1], data[2])
        elif stype == 'setSearchErrmsg':
            self.view.setSearchErrmsg(data)

    def __startThread__(self, index: int):
        def __thread_search__(model: SearchModel, index: int):
            typeIndex = model.view.getSelectedTabIndex()
            searchText = model.view.getSearchText()

            if aigpy.stringHelper.isNull(searchText):
                model._lock.release()
                return

            # search
            limit = 20
            offset = (index - 1) * limit
            stype = tidal_dl.Type(typeIndex)
            msg, model._resultData = API.search(searchText, stype, offset, limit)

            if not aigpy.stringHelper.isNull(msg):
                model.SIGNAL_REFRESH_VIEW.emit('setSearchErrmsg', msg)
                model._lock.release()
                return

            # get page index
            total = model.__getSumByResult__(stype)
            if total <= 0:
                model.SIGNAL_REFRESH_VIEW.emit('setSearchErrmsg', 'Search results are empty...')
                model._lock.release()
                return

            maxIdx = total // limit + (1 if total % limit > 0 else 0)
            if index > maxIdx:
                model._lock.release()
                return

            # set view
            model.SIGNAL_REFRESH_VIEW.emit('setPageIndex', (index, maxIdx))
            model.SIGNAL_REFRESH_VIEW.emit('setTableItems', (stype, offset, model._resultData))
            model._lock.release()

        _thread.start_new_thread(__thread_search__, (self, index))

    def __search__(self, num: int = 0):
        if not self._lock.acquire(False):
            return

        self.view.setSearchErrmsg('')

        if aigpy.string.isNull(self.view.getSearchText()):
            self._lock.release()
            return

        index = 1
        if num != 0:
            curIdx, curSum = self.view.getPageIndex()
            if curIdx + num > 1:
                index = curIdx + num

        self.__startThread__(index)

    def __searchNext__(self):
        self.__search__(1)

    def __searchPre__(self):
        self.__search__(-1)

    def __download__(self):
        if self._resultData is None:
            self.view.setSearchErrmsg('Please search first...')
            return

        typeIndex = self.view.getSelectedTabIndex()
        stype = tidal_dl.Type(typeIndex)

        items = []
        if stype == Type.Album:
            items = self._resultData.albums.items
        elif stype == Type.Track:
            items = self._resultData.tracks.items
        elif stype == Type.Video:
            items = self._resultData.videos.items
        elif stype == Type.Playlist:
            items = self._resultData.playlists.items

        index = self.view.getSelectedTableIndex(stype)
        if index < 0 or index >= len(items):
            self.view.setSearchErrmsg('Please select one item...')
            return

        self.SIGNAL_ADD_TASKITEM.emit(items[index])

    def __changeAudioQuality__(self):
        setCurAudioQuality(self.view.getTrackQualityText())
    
    def __changeVideoQuality__(self):
        setCurVideoQuality(self.view.getVideoQualityText())



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  main.py
@Date    :  2021/05/11
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from enum import IntEnum

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QVBoxLayout, QWidget, QGridLayout

from tidal_gui.control.framelessWidget import FramelessWidget
from tidal_gui.control.pushButton import PushButton
from tidal_gui.style import ButtonStyle


class PageType(IntEnum):
    Search = 0,
    Task = 1,
    Settings = 2,
    About = 3,
    Max = 4,


class MainView(FramelessWidget):
    viewWidth = 1200
    viewHeight = 700

    def __init__(self):
        super(MainView, self).__init__()
        self.setMinimumHeight(self.viewHeight)
        self.setMinimumWidth(self.viewWidth)
        self.__initPages__()
        self.__initView__()
        self.setWindowButton(True, True, True)

    def __initPages__(self):
        self._pages = []
        for index in range(0, PageType.Max):
            self._pages.append(None)

    def __initView__(self):
        leftTab = self.__initLeftTab__()
        content = self.__initContent__()
        moveWgt = self.__initMoveHead__()

        grid = self.getGrid()
        grid.addWidget(leftTab, 0, 0, 2, 1, Qt.AlignLeft)
        grid.addWidget(moveWgt, 0, 1, Qt.AlignTop)
        grid.addLayout(content, 1, 1)

        self.setValidMoveWidget(moveWgt)

    def __initLeftTab__(self):
        self._icon = PushButton("", ButtonStyle.SoftwareIcon)
        self._searchBtn = PushButton("", ButtonStyle.SearchIcon)
        self._taskBtn = PushButton("", ButtonStyle.TaskIcon)
        self._settingsBtn = PushButton("", ButtonStyle.SettingsIcon)
        self._aboutBtn = PushButton("", ButtonStyle.AboutIcon)

        self._searchBtn.clicked.connect(lambda: self.showPage(PageType.Search))
        self._taskBtn.clicked.connect(lambda: self.showPage(PageType.Task))
        self._settingsBtn.clicked.connect(lambda: self.showPage(PageType.Settings))
        self._aboutBtn.clicked.connect(lambda: self.showPage(PageType.About))

        layout = QVBoxLayout()
        layout.setSpacing(15)
        layout.setContentsMargins(15, 45, 15, 20)
        layout.addWidget(self._icon)
        layout.addWidget(self._searchBtn)
        layout.addWidget(self._taskBtn)
        layout.addStretch(1)
        # layout.addWidget(self._settingsBtn)
        # layout.addWidget(self._aboutBtn)

        widget = QWidget()
        widget.setLayout(layout)
        widget.setObjectName("MainViewLeftWidget")
        return widget

    def __initMoveHead__(self) -> QWidget:
        self._moveWidget = QWidget()
        self._moveWidget.setFixedHeight(30)
        self._moveWidget.setObjectName("BaseWidget")
        return self._moveWidget

    def __initContent__(self) -> QGridLayout:
        self._searchView = None
        self._settingsView = None
        self._aboutView = None
        self._taskView = None
        self._contentLayout = QGridLayout()
        self._contentLayout.setContentsMargins(10, 0, 10, 10)
        return self._contentLayout

    def showPage(self, pageType: PageType = PageType.Search):
        for index in range(0, PageType.Max):
            if index == pageType:
                self._pages[index].show()
            else:
                self._pages[index].hide()

    def __setContentPage__(self, view, pageType: PageType):
        self._pages[pageType] = view
        self._pages[pageType].hide()
        self._contentLayout.addWidget(view, 0, 0)

    def setSearchView(self, view):
        self.__setContentPage__(view, PageType.Search)

    def setTaskView(self, view):
        self.__setContentPage__(view, PageType.Task)

    def setSettingsView(self, view):
        self.__setContentPage__(view, PageType.Settings)

    def setAboutView(self, view: QWidget):
        self._pages[PageType.About] = view
        self._pages[PageType.About].hide()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  tableView.py
@Date    :  2021/9/10
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtWidgets import QTableView, QAbstractItemView


class TableView(QTableView):
    def __init__(self, columnNames: list, rowCount: int = 20):
        super(TableView, self).__init__()

        self._model = QStandardItemModel()
        self._model.setColumnCount(len(columnNames))
        self._model.setRowCount(rowCount)

        for index, name in enumerate(columnNames):
            self._model.setHeaderData(index, Qt.Horizontal, name)

        self.setModel(self._model)
        # self.setHorizontalHeaderItem(index, QTableWidgetItem(name))
        # for index in range(0, rowCount):
        #     self.setRowHeight(index, 50)

        self.setShowGrid(False)
        self.verticalHeader().setVisible(False)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setSelectionMode(QAbstractItemView.SingleSelection)

        self.horizontalHeader().setStretchLastSection(True)
        self.setEditTriggers(QAbstractItemView.NoEditTriggers)

        self.setFocusPolicy(Qt.NoFocus)

    def addItem(self, rowIdx: int, colIdx: int, text: str):
        item = QStandardItem(text)
        item.setTextAlignment(Qt.AlignCenter)
        self._model.setItem(rowIdx, colIdx, item)
    #
    # def addWidgetItem(self, rowIdx: int, colIdx: int, widget):
    #     self.setCellWidget(rowIdx, colIdx, widget)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  aboutView.py
@Date    :  2021/8/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QGridLayout, QHBoxLayout, QLabel

from tidal_gui.control.label import Label
from tidal_gui.control.pushButton import PushButton
from tidal_gui.style import LabelStyle, ButtonStyle
from tidal_gui.theme import getResourcePath


class AboutView(QWidget):
    def __init__(self):
        super(AboutView, self).__init__()
        self.__initView__()

    def __initView__(self):
        grid = QGridLayout(self)
        grid.addWidget(self.__initLogo__(), 0, 0, Qt.AlignLeft)
        grid.addLayout(self.__initContent__(), 0, 1)

    def __initLogo__(self):
        path = getResourcePath() + "/svg/V.svg"
        self._logo = QLabel()
        self._logo.setPixmap(QPixmap(path))
        return self._logo

    def __initButton__(self):
        path = getResourcePath() + "/svg/"

        self._feedbackBtn = PushButton('Feedback', ButtonStyle.Default, iconUrl=path + 'github.svg')
        self._buymeacoffeeBtn = PushButton('Buymeacoffee', ButtonStyle.Info, iconUrl=path + 'buymeacoffee.svg')
        self._paypalBtn = PushButton('Paypal', ButtonStyle.Primary, iconUrl=path + 'paypal.svg')

        layout = QHBoxLayout()
        layout.addWidget(self._feedbackBtn)
        layout.addWidget(self._buymeacoffeeBtn)
        layout.addWidget(self._paypalBtn)
        return layout

    def __initContent__(self):
        self._titleLabel = Label('', LabelStyle.HugeTitle)
        self._authorLabel = Label('')
        self._versionLabel = Label('')
        self._lastVersionLabel = Label('')

        layout = QVBoxLayout()
        layout.addWidget(self._titleLabel)
        layout.addWidget(self._authorLabel)
        layout.addWidget(self._versionLabel)
        layout.addWidget(self._lastVersionLabel)
        layout.addLayout(self.__initButton__())
        return layout

    def setTitle(self, text: str):
        self._titleLabel.setText(text)

    def setAuthor(self, text: str):
        self._authorLabel.setText('MADE WITH ♥ BY ' + text)

    def setVersion(self, text: str):
        self._versionLabel.setText('VERSION ' + text)

    def setLastVersion(self, text: str):
        self._lastVersionLabel.setText('LAST-VERSION ' + text)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  pushButton.py
@Date    :  2021/05/08
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QPushButton

from tidal_gui.style import ButtonStyle


class PushButton(QPushButton):
    def __init__(self,
                 text: str = '',
                 style: ButtonStyle = ButtonStyle.Default,
                 width=0,
                 iconUrl=''):
        super(PushButton, self).__init__()
        self.setText(text)
        self.setObjectName(style.name + "PushButton")

        if width > 0:
            self.setFixedWidth(width)

        if iconUrl != '':
            self.setIcon(QIcon(iconUrl))



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  taskModel.py
@Date    :  2021/8/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
import threading
from PyQt5.QtCore import QTimer
from tidal_dl.model import Album, Artist
from tidal_gui.view.taskView import TaskView, TaskStatus
from tidal_gui.viewModel.downloadItemModel import DownloadItemModel
from tidal_gui.viewModel.taskItemModel import TaskItemModel, TaskStatus
from tidal_gui.viewModel.viewModel import ViewModel


class TaskModel(ViewModel):
    def __init__(self):
        super(TaskModel, self).__init__()
        self.view = TaskView()

        self._listMap = {}
        for item in map(lambda typeItem: typeItem.name, TaskStatus):
            self._listMap[item] = []

        self._timer = QTimer(self)
        self._timer.timeout.connect(self.__checkTaskStatus__)
        self._timer.start(3000)
        
        # self.test()
        
    def __checkTaskStatus__(self):
        for item in self._listMap[TaskStatus.Download.name][:]:
            status = item.getTaskStatus()
            if status == TaskStatus.Download:
                continue
            
            self._listMap[TaskStatus.Download.name].remove(item)
            self.view.delItemView(TaskStatus.Download, item.view)

            self._listMap[status.name].append(item)
            self.view.addItemView(status, item.view)

    def uninit(self):
        self._timer.stop()
        for item in self._listMap[TaskStatus.Download.name]:
            for downItem in item.downloadModelList:
                downItem.stopDownload()

    def addTaskItem(self, data):
        item = TaskItemModel(data)
        self._listMap[TaskStatus.Download.name].append(item)
        self.view.addItemView(TaskStatus.Download, item.view)

    def getWaitDownloadItem(self) -> DownloadItemModel:
        for item in self._listMap[TaskStatus.Download.name]:
            for downItem in item.downloadModelList:
                if downItem.isInWait():
                    return downItem
        return None

    def test(self):
        ar = Artist()
        ar.name = 'yaron'
        ar.id = 110
        album = Album()
        album.artist = None
        album.artists = [ar]
        album.audioModes = ['STEREO']
        album.audioQuality = 'LOSSLESS'
        album.cover = '8203ff9a-47e3-49a0-9b44-a6dbbe9c9469'
        album.duration = 1140
        album.explicit = False
        album.id = 177748204
        album.numberOfTracks = 10
        album.numberOfVideos = 0
        album.numberOfVolumes = 1
        album.releaseDate = '2021-03-16'
        album.title = 'Love'
        album.type = 'ALBUM'
        album.version = None

        self.addTaskItem(album)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  settingsView.py
@Date    :  2021/05/11
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QGridLayout

from tidal_gui.control.checkBox import CheckBox
from tidal_gui.control.comboBox import ComboBox
from tidal_gui.control.label import Label
from tidal_gui.control.line import Line
from tidal_gui.control.lineEdit import LineEdit
from tidal_gui.control.pushButton import PushButton
from tidal_gui.style import LabelStyle, ButtonStyle, ThemeStyle


class SettingsView(QWidget):
    def __init__(self):
        super(SettingsView, self).__init__()
        self.__initView__()

    def __initView__(self):
        grid = QVBoxLayout(self)
        grid.addLayout(self.__initHeader__())
        grid.addLayout(self.__initContent__())
        grid.addLayout(self.__initToolButton__())

    def __initHeader__(self):
        self._header = Label("SETTINGS", LabelStyle.PageTitle)
        layout = QVBoxLayout()
        layout.addWidget(self._header)
        layout.addWidget(Line('H'))
        layout.setSpacing(10)
        layout.setContentsMargins(0, 0, 0, 10)
        return layout

    def __initToolButton__(self) -> QHBoxLayout:
        self._logoutBtn = PushButton('Logout', ButtonStyle.Danger, 100)
        self._cancelBtn = PushButton('Cancel', ButtonStyle.Default, 100)
        self._confirmBtn = PushButton('OK', ButtonStyle.Primary, 100)

        layout = QHBoxLayout()
        layout.addWidget(self._logoutBtn)
        layout.addStretch(1)
        layout.addWidget(self._cancelBtn)
        layout.addWidget(self._confirmBtn)
        return layout

    def __addContent__(self, control, desc=''):
        rowIdx = self._contentLayout.rowCount()
        if desc != '':
            self._contentLayout.addWidget(Label(desc), rowIdx, 0, Qt.AlignRight)
        self._contentLayout.addWidget(control, rowIdx, 1)

    def __initContent__(self):
        self._contentLayout = QGridLayout()
        self._contentLayout.setSpacing(15)

        self._pathEdit = LineEdit()
        self._threadNumComboBox = ComboBox([1, 5, 10, 20])
        self._searchNumComboBox = ComboBox([10, 20, 30, 40, 50])
        self.__addContent__(self._pathEdit, 'Path:')
        self.__addContent__(self._threadNumComboBox, 'ThreadNum:')
        self.__addContent__(self._searchNumComboBox, 'SearchNum:')

        self._contentLayout.setRowStretch(self._contentLayout.rowCount(), 1)

        self._albumFolderFormatEdit = LineEdit()
        self._trackFileFormatEdit = LineEdit()
        self._videoFileFormatEdit = LineEdit()
        self.__addContent__(self._albumFolderFormatEdit, 'AlbumFolderFormat:')
        self.__addContent__(self._trackFileFormatEdit, 'TrackFileFormat:')
        self.__addContent__(self._videoFileFormatEdit, 'VideoFileFormat:')

        self._saveCoverCheck = CheckBox('Download album cover')
        self._skipExistCheck = CheckBox('Skip exist file when downloading')
        self._convertMp4ToM4a = CheckBox('Convert mp4 to m4a')
        self.__addContent__(self._saveCoverCheck)
        self.__addContent__(self._skipExistCheck)
        self.__addContent__(self._convertMp4ToM4a)

        self._contentLayout.setRowStretch(self._contentLayout.rowCount(), 1)

        self._themeComboBox = ComboBox(list(map(lambda x: x.name, ThemeStyle)))
        self._languageComboBox = ComboBox(['Default'])
        self.__addContent__(self._themeComboBox, 'Theme:')
        self.__addContent__(self._languageComboBox, 'Language:')

        self._contentLayout.setRowStretch(self._contentLayout.rowCount(), 10)
        return self._contentLayout

    def connectButton(self, name: str, func):
        if name == "Logout":
            self._logoutBtn.clicked.connect(func)
        elif name == "Cancel":
            self._cancelBtn.clicked.connect(func)
        elif name == "OK":
            self._confirmBtn.clicked.connect(func)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  settingsModel.py
@Date    :  2021/8/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from tidal_gui.view.settingsView import SettingsView
from tidal_gui.viewModel.viewModel import ViewModel


class SettingsModel(ViewModel):
    def __init__(self, parent):
        super(SettingsModel, self).__init__()
        self._parent = parent
        self.view = SettingsView()
        self.view.connectButton('Logout', self.__logout__)
        self.view.connectButton('Cancel', self.__cancel__)
        self.view.connectButton('OK', self.__ok__)

    def __logout__(self):
        self._parent.show(True)

    def __cancel__(self):
        pass

    def __ok__(self):
        pass



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  qssParse.py
@Date    :  2021/05/08
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""

import os

import aigpy

from tidal_gui.style import ThemeStyle

_RESOURCE_PATH = './resource'
if os.path.isdir(_RESOURCE_PATH):
    _RESOURCE_PATH = os.path.abspath(_RESOURCE_PATH).replace('\\', '/')
else:
    _RESOURCE_PATH = aigpy.path.getDirName(__file__).replace('\\', '/') + "resource"


def __getParam__(line: str):
    key = aigpy.string.getSub(line, "--", ":")
    value = aigpy.string.getSub(line, ":", ";")
    return key, value


def __parseParamsList__(content: str) -> dict:
    globalStr = aigpy.string.getSub(content, ":root", "}")
    lines = globalStr.split("\n")

    array = {}
    for line in lines:
        key, value = __getParam__(line)
        if key == "" or value == "":
            continue
        array[key] = value
    return array


def __parseQss__(content: str, params: dict) -> str:
    content = aigpy.string.getSub(content, "/* #QSS_START  */")
    for key in params:
        content = content.replace("var(--" + key + ")", params[key])

    content = content.replace("$RESOURCE_PATH$", _RESOURCE_PATH)
    return content


def __getQss__(filePath: str) -> str:
    content = aigpy.file.getContent(filePath)
    params = __parseParamsList__(content)
    qss = __parseQss__(content, params)
    return qss


def getResourcePath():
    return _RESOURCE_PATH


def getThemeQssContent(style: ThemeStyle = ThemeStyle.Default):
    name = "theme" + style.name + ".qss"
    return __getQss__(_RESOURCE_PATH + '/' + name)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  loginView.py
@Date    :  2021/04/30
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import *

from tidal_gui.control.checkBox import CheckBox
from tidal_gui.control.framelessWidget import FramelessWidget
from tidal_gui.control.label import Label
from tidal_gui.control.layout import createHBoxLayout
from tidal_gui.control.lineEdit import LineEdit
from tidal_gui.control.pushButton import PushButton
from tidal_gui.style import ButtonStyle, LabelStyle
from tidal_gui.theme import getResourcePath


class LoginView(FramelessWidget):
    viewWidth = 650
    viewHeight = 400
    logoWidth = 300

    def __init__(self):
        super(LoginView, self).__init__()
        self.setFixedSize(self.viewWidth, self.viewHeight)
        self.__initView__()
        self.setWindowButton(True, False, False)

    def __initAccountTab__(self):
        self._deviceCodeEdit = LineEdit()
        self._confirmBtn = PushButton("LOGIN", ButtonStyle.Primary)

        grid = QGridLayout()
        grid.setSpacing(15)
        grid.setRowStretch(0, 1)

        grid.addLayout(createHBoxLayout([Label("DeviceCode"), self._deviceCodeEdit]), 1, 0, 1, 2)
        grid.setRowStretch(3, 1)
        grid.addWidget(self._confirmBtn, 5, 0, 1, 2)
        grid.setRowStretch(6, 1)

        widget = QWidget()
        widget.setLayout(grid)
        return widget

    def __initProxyTab__(self):
        self._enableProxyCheck = CheckBox('')
        self._proxyHostEdit = LineEdit()
        self._proxyPortEdit = LineEdit()
        self._proxyUserEdit = LineEdit()
        self._proxyPwdEdit = LineEdit()

        grid = QGridLayout()
        grid.setSpacing(8)
        grid.setRowStretch(0, 1)
        grid.addWidget(Label("HttpProxy"), 1, 0)
        grid.addWidget(self._enableProxyCheck, 1, 1)
        grid.addWidget(Label("Host"), 2, 0)
        grid.addWidget(self._proxyHostEdit, 2, 1)
        grid.addWidget(Label("Port"), 3, 0)
        grid.addWidget(self._proxyPortEdit, 3, 1)
        grid.addWidget(Label("UserName"), 4, 0)
        grid.addWidget(self._proxyUserEdit, 4, 1)
        grid.addWidget(Label("Password"), 5, 0)
        grid.addWidget(self._proxyPwdEdit, 5, 1)
        grid.setRowStretch(6, 1)

        widget = QWidget()
        widget.setLayout(grid)
        return widget

    def __initIconWidget__(self):
        self._icon = Label('')
        self._icon.setStyleSheet("QLabel{background-color:rgb(0,0,0);}")

        self._icon.setPixmap(QPixmap(getResourcePath() + "/svg/V.svg"))
        self._icon.setAlignment(Qt.AlignCenter)

        self._iconLabel = Label('', LabelStyle.LogoBottom)
        self._iconLabel.setAlignment(Qt.AlignCenter)
        self._iconLabel.hide()

        layout = QVBoxLayout()
        layout.setSpacing(15)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addStretch(1)
        layout.addWidget(self._icon)
        layout.addWidget(self._iconLabel)
        layout.addStretch(1)

        widget = QWidget()
        widget.setStyleSheet("QWidget{background-color:rgb(0,0,0);}")
        widget.setLayout(layout)
        return widget

    def __initView__(self):
        iconWidget = self.__initIconWidget__()

        self._tab = QTabWidget(self)
        self._tab.addTab(self.__initAccountTab__(), "LOGIN")
        self._tab.addTab(self.__initProxyTab__(), "PROXY")
        self._tab.setFixedWidth(self.viewWidth - self.logoWidth - 60)
        self._tab.hide()

        grid = self.getGrid()
        grid.setSpacing(15)
        grid.setContentsMargins(0, 0, 0, 0)
        grid.addWidget(iconWidget)
        grid.addWidget(self._tab, 0, 1, Qt.AlignCenter)

        self.setValidMoveWidget(iconWidget)

    def showEnterView(self):
        self._tab.show()

    def hideEnterView(self):
        self._tab.hide()

    def setDeviceCode(self, text):
        self._deviceCodeEdit.setText(text)

    def enableHttpProxy(self) -> bool:
        return self._enableProxyCheck.isChecked()

    def getProxyInfo(self) -> dict:
        infos = {'host': self._proxyHostEdit.text(), 'port': self._proxyPortEdit.text(),
                 'username': self._proxyUserEdit.text(), 'password': self._proxyPwdEdit.text()}
        return infos

    def connectConfirmButton(self, func):
        self._confirmBtn.clicked.connect(func)

    def enableConfirmButton(self, enable):
        self._confirmBtn.setEnabled(enable)

    def setMsg(self, text):
        if len(text) <= 0:
            self._iconLabel.hide()
        else:
            self._iconLabel.setText(text)
            self._iconLabel.show()

    def showErrMessage(self, text: str):
        qmb = QMessageBox(self)
        qmb.setWindowTitle('Error')
        qmb.setIcon(QMessageBox.Warning)
        qmb.setText(text)
        qmb.addButton(QPushButton('OK', qmb), QMessageBox.YesRole)
        qmb.open()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :  enum.py
@Date    :  2021/05/08
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :  
'''

from enum import Enum


class ButtonStyle(Enum):
    Default = 0,
    Primary = 1,
    Success = 2,
    Danger = 3,
    Warning = 4,
    Info = 5,

    CloseWindow = 6,
    MaxWindow = 7,
    MinWindow = 8,

    SearchIcon = 9,
    TaskIcon = 10,
    SettingsIcon = 11,
    AboutIcon = 12,

    SoftwareIcon = 13

    PrePage = 14
    NextPage = 15

    TaskRetry = 16,
    TaskCancel = 17,
    TaskDelete = 18,
    TaskOpen = 19,
    TaskExpand = 20,


class LabelStyle(Enum):
    Default = 0,
    PageTitle = 1,
    PageSubTitle = 2,
    HugeTitle = 3,
    LogoBottom = 4,
    SearchErr = 5,
    Icon = 6,
    Bold = 7,
    Italic = 9,
    Tag = 10


class ThemeStyle(Enum):
    Default = 0,
    Dark = 1,


class ListWidgetStyle(Enum):
    Default = 0,
    TaskTab = 1,
    DownloadItems = 2,



from setuptools import setup, find_packages
setup(
    name = 'tidal-gui',
    version = '2022.01.18.3',
    license = "Apache2",
    description = "Tidal Music Downloader-GUI.",

    author = 'YaronH',
    author_email = "yaronhuang@foxmail.com",

    packages=find_packages(exclude=['tidal_dl*']),
    include_package_data = True,
    platforms = "any",
    install_requires=["tidal-dl", "PyQt5"],
    entry_points={'console_scripts': [ 'tidal-gui = tidal_gui:main', ]}
)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  downloadItemModel.py
@Date    :  2021/10/08
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :  
"""

import os
import aigpy
from abc import ABC, ABCMeta
from enum import Enum
from pickle import FALSE
from aigpy.downloadHelper import UserProgress
import aigpy.stringHelper
from PyQt5.QtCore import pyqtSignal

from tidal_dl.model import Track
from tidal_dl.util import downloadTrack, downloadVideo, getArtistsNames, setMetaData
from tidal_gui.view.downloadItemView import DownloadItemView
from tidal_gui.viewModel.viewModel import ViewModel


class DownloadStatus(Enum):
    WAIT = 0,
    RUNNING = 1,
    SUCCESS = 2,
    ERROR = 3,
    CANCEL = 4,


_endStatus_ = [DownloadStatus.SUCCESS, DownloadStatus.ERROR, DownloadStatus.CANCEL]


class Progress(UserProgress):
    def __init__(self, model):
        super().__init__()
        self.model = model
        self.curStr = ''
        self.maxStr = ''

    def __toMBStr__(self, num):
        size = aigpy.memory.convert(num, aigpy.memory.Unit.BYTE, aigpy.memory.Unit.MB)
        return str(round(size, 2)) + ' MB'

    def updateCurNum(self):
        per = self.curNum * 100 / self.maxNum
        self.curStr = self.__toMBStr__(self.curNum)
        self.model.SIGNAL_REFRESH_VIEW.emit('updateCurNum', {'per': per,
                                                             'curStr': self.curStr,
                                                             'maxStr': self.maxStr})

    def updateMaxNum(self):
        self.maxStr = self.__toMBStr__(self.maxNum)
        
    def updateStream(self, stream):
        self.model.SIGNAL_REFRESH_VIEW.emit('updateStream', {'stream': stream})


class DownloadItemModel(ViewModel):
    SIGNAL_END = pyqtSignal(DownloadStatus)
    
    def __init__(self, index, data, basePath):
        super(DownloadItemModel, self).__init__()
        self.view = DownloadItemView()
        self.data = data
        self.basePath = basePath
        self.isTrack = isinstance(data, Track)
        self.progress = Progress(self)
        self.__setStatus__(DownloadStatus.WAIT)

        if self.isTrack:
            self.__initTrack__(index)
        else:
            self.__initVideo__(index)

        self.SIGNAL_REFRESH_VIEW.connect(self.__refresh__)

    def __refresh__(self, stype: str, object):
        if stype == "updateCurNum":
            per = object['per']
            curStr = object['curStr']
            maxStr = object['maxStr']
            self.view.setSize(curStr, maxStr)
            self.view.setProgress(per)
        elif stype == "updateStream":
            codec = object['stream'].codec
            self.view.setCodec(codec)


    def __setStatus__(self, status: DownloadStatus, desc: str = ''):
        self.status = status
        if desc == '':
            self.view.setAction(status.name)
        else:
            self.view.setAction(status.name + '-' + desc)
        if status in _endStatus_:
            self.SIGNAL_END.emit(status)
        

    def __setErrStatus__(self, errmsg: str):
        self.view.setErrmsg(errmsg)
        self.__setStatus__(DownloadStatus.ERROR)

    def __initTrack__(self, index):
        title = self.data.title
        own = self.data.album.title
        self.view.setLabel(index, title, own)

    def __initVideo__(self, index):
        title = self.data.title
        own = getArtistsNames(self.data.artists)
        self.view.setLabel(index, title, own)

    def isInWait(self):
        return self.status == DownloadStatus.WAIT

    def stopDownload(self):
        if self.status not in _endStatus_:
            self.__setStatus__(DownloadStatus.CANCEL)

    def retry(self):
        if self.status in [DownloadStatus.ERROR, DownloadStatus.CANCEL]:
            self.__setStatus__(DownloadStatus.WAIT)

    def download(self):
        self.__setStatus__(DownloadStatus.RUNNING)

        if self.isTrack:
            check, msg = downloadTrack(self.data, self.data.album, self.data.playlist, self.progress)
        else:
            check, msg = downloadVideo(self.data)

        if check is False:
            self.__setErrStatus__(msg)
            return

        self.view.setProgress(100)
        self.__setStatus__(DownloadStatus.SUCCESS)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  framelessWidget.py
@Date    :  2021/05/08
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""

from PyQt5.QtCore import Qt, QPoint
from PyQt5.QtGui import QMouseEvent
from PyQt5.QtWidgets import QWidget, QGridLayout, QHBoxLayout

from tidal_gui.control.pushButton import PushButton
from tidal_gui.style import ButtonStyle


class FramelessWidget(QWidget):
    def __init__(self):
        super(FramelessWidget, self).__init__()
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.BorderWidth = 5
        self.borderWidget = QWidget()
        self.borderWidget.setObjectName("widgetMain")
        self.borderWidget.setStyleSheet("QWidget#widgetMain{border: 1px solid #000000;};")

        self.contentGrid = QGridLayout()
        self.contentGrid.setContentsMargins(1, 1, 1, 1)

        self.windowBtnGrid = self.__createWindowsButtonLayout__()
        self.enableMove = True
        self.validMoveWidget = None
        self.clickPos = None

        self.grid = QGridLayout()
        self.grid.setSpacing(0)
        self.grid.setContentsMargins(0, 0, 0, 0)
        self.grid.addWidget(self.borderWidget, 0, 0)
        self.grid.addLayout(self.contentGrid, 0, 0)
        self.setLayout(self.grid)

    def __showMaxWindows__(self):
        if self.windowState() == Qt.WindowMaximized:
            self.showNormal()
        else:
            self.showMaximized()

    def __createWindowsButtonLayout__(self):
        self.closeBtn = PushButton('', ButtonStyle.CloseWindow)
        self.maxBtn = PushButton('', ButtonStyle.MaxWindow)
        self.minBtn = PushButton('', ButtonStyle.MinWindow)

        self.closeBtn.clicked.connect(self.close)
        self.minBtn.clicked.connect(self.showMinimized)
        self.maxBtn.clicked.connect(self.__showMaxWindows__)

        layout = QHBoxLayout()
        layout.setSpacing(0)
        layout.setContentsMargins(1, 1, 1, 1)
        layout.addWidget(self.minBtn)
        layout.addWidget(self.maxBtn)
        layout.addWidget(self.closeBtn)
        return layout

    def __clickInValidMoveWidget__(self, x=-1, y=-1) -> bool:
        if self.validMoveWidget is None:
            return False
        if self.clickPos is None:
            return False
        
        if x == -1 and y == -1:
            x = self.clickPos.x()
            y = self.clickPos.y()

        pos = self.validMoveWidget.pos()
        if x < pos.x() or x > pos.x() + self.validMoveWidget.width():
            return False
        if y < pos.y() or y > pos.y() + self.validMoveWidget.height():
            return False
        return True

    def nativeEvent(self, eventType, message):
        retVal, result = super(FramelessWidget, self).nativeEvent(eventType, message)
        # if eventType == "windows_generic_MSG":
        #     msg = ctypes.wintypes.MSG.from_address(message.__int__())
        #     if msg.message != win32con.WM_NCHITTEST:
        #         return retVal, result

        #     # 获取鼠标移动经过时的坐标
        #     x = win32api.LOWORD(msg.lParam) - self.frameGeometry().x()
        #     y = win32api.HIWORD(msg.lParam) - self.frameGeometry().y()

        #     w, h = self.width(), self.height()
        #     lx = x < self.BorderWidth
        #     rx = x > w - self.BorderWidth
        #     ty = y < self.BorderWidth
        #     by = y > h - self.BorderWidth
        #     if (lx and ty):# 左上角
        #         return True, win32con.HTTOPLEFT
        #     elif (rx and by):# 右下角
        #         return True, win32con.HTBOTTOMRIGHT
        #     elif (rx and ty):# 右上角
        #         return True, win32con.HTTOPRIGHT
        #     elif (lx and by):# 左下角
        #         return True, win32con.HTBOTTOMLEFT
        #     elif ty:# 上
        #         return True, win32con.HTTOP
        #     elif by:# 下
        #         return True, win32con.HTBOTTOM
        #     elif lx:# 左
        #         return True, win32con.HTLEFT
        #     elif rx:# 右
        #         return True, win32con.HTRIGHT
        return retVal, result

    def mousePressEvent(self, e: QMouseEvent):
        if e.button() == Qt.LeftButton:
            self.clickPos = e.pos()

    def mouseReleaseEvent(self, e: QMouseEvent):
        if e.button() == Qt.LeftButton:
            self.clickPos = QPoint(-1, -1)

    def mouseMoveEvent(self, e: QMouseEvent):
        if not self.enableMove:
            return
        if Qt.LeftButton & e.buttons():
            if self.__clickInValidMoveWidget__() and self.clickPos:
                self.move(e.pos() + self.pos() - self.clickPos)

    def mouseDoubleClickEvent(self, e: QMouseEvent):
        if self.maxBtn.isHidden():
            return
        if Qt.LeftButton & e.buttons():
            if self.__clickInValidMoveWidget__(e.x(), e.y()):
                self.__showMaxWindows__()

    def getGrid(self):
        return self.contentGrid

    def disableMove(self):
        self.enableMove = False

    def setValidMoveWidget(self, widget):
        self.validMoveWidget = widget

    def setWindowButton(self, showClose=True, showMin=True, showMax=True):
        if not showMax:
            self.maxBtn.hide()
        if not showMin:
            self.minBtn.hide()
        if not showClose:
            self.closeBtn.hide()
        self.grid.addLayout(self.windowBtnGrid, 0, 0, Qt.AlignTop | Qt.AlignRight)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  label.py
@Date    :  2021/05/08
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from PyQt5.QtWidgets import QLabel

from tidal_gui.style import LabelStyle


class Label(QLabel):
    def __init__(self, text: str = "", style: LabelStyle = LabelStyle.Default):
        super(Label, self).__init__()
        self.setText(text)
        self.setObjectName(style.name + "Label")



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  viewModel.py
@Date    :  2021/8/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from PyQt5.QtCore import QObject
from PyQt5.QtCore import pyqtSignal


class ViewModel(QObject):
    SIGNAL_REFRESH_VIEW = pyqtSignal(str, object)

    def __init__(self):
        super(ViewModel, self).__init__()
        self.view = None

    def show(self):
        self.view.show()

    def hide(self):
        self.view.hide()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  downloadItemView.py
@Date    :  2021/10/08
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :  
"""
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QWidget, QHBoxLayout, QGridLayout, QProgressBar

from tidal_gui.control.label import Label
from tidal_gui.style import LabelStyle


class DownloadItemView(QWidget):
    def __init__(self):
        super(DownloadItemView, self).__init__()
        self.__initView__()
        self.setObjectName('DownloadItemView')
        self.setAttribute(Qt.WA_StyledBackground)

    def __initView__(self):
        self._indexLabel = Label('1')
        self._codecLabel = Label('', LabelStyle.Tag)
        self._titleLabel = Label('title', LabelStyle.Bold)
        self._ownLabel = Label('own', LabelStyle.Italic)
        self._ownLabel.setMaximumWidth(200)
        self._actionLabel = Label('', LabelStyle.Italic)
        self._actionLabel.setFixedWidth(80)
        
        self._errLabel = Label('')
        self._errLabel.setVisible(False)

        self._sizeLabel = Label('/')
        self._speedLabel = Label('')

        self._progress = QProgressBar()
        self._progress.setTextVisible(False)
        self._progress.setFixedHeight(3)
        self._progress.setFixedWidth(300)
        self._progress.setRange(0, 100)

        titleLayout = QHBoxLayout()
        titleLayout.setSpacing(3)
        titleLayout.setContentsMargins(0, 0, 0, 0)
        titleLayout.addWidget(self._indexLabel, Qt.AlignLeft)
        titleLayout.addWidget(self._titleLabel, Qt.AlignLeft)
        titleLayout.addStretch(50)
        titleLayout.addWidget(self._codecLabel, Qt.AlignRight)
        titleLayout.addWidget(self._ownLabel, Qt.AlignRight)
        
        speedLayout = QHBoxLayout()
        speedLayout.setSpacing(30)
        speedLayout.addWidget(self._sizeLabel)
        speedLayout.addWidget(self._speedLabel)

        grid = QGridLayout(self)
        grid.setContentsMargins(0,0,0,0)
        grid.setSpacing(2)
        grid.addLayout(titleLayout, 0, 0, Qt.AlignLeft | Qt.AlignVCenter)
        grid.addWidget(self._progress, 0, 1, Qt.AlignRight | Qt.AlignVCenter)
        grid.addWidget(self._actionLabel, 0, 2, Qt.AlignRight | Qt.AlignVCenter)
        grid.addWidget(self._errLabel, 1, 0, Qt.AlignLeft | Qt.AlignVCenter)
        grid.addLayout(speedLayout, 1, 1, Qt.AlignLeft | Qt.AlignVCenter)

    def setLabel(self, index, title, own):
        self._indexLabel.setText(str(index))
        self._titleLabel.setText(title)
        self._ownLabel.setText(own)

    def setErrmsg(self, msg):
        self._errLabel.setText(msg)
        self._errLabel.setVisible(len(msg) > 0)

    def setAction(self, msg):
        self._actionLabel.setText(msg)

    def setProgress(self, value):
        self._progress.setValue(value)
        pass

    def setSize(self, curSize: str, totalSize: str):
        self._sizeLabel.setText(f'{curSize} / {totalSize}')

    def setSpeed(self, speed: str):
        self._speedLabel.setText(speed)

    def setCodec(self, codec: str):
        self._codecLabel.setText(codec)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  scrollWidget.py
@Date    :  2021/10/08
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :  
"""
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QWidget, QScrollArea, QVBoxLayout
from PyQt5.QtGui import QResizeEvent


class ScrollWidget(QScrollArea):
    def __init__(self):
        super(ScrollWidget, self).__init__()
        self._numWidget = 0
        self._layout = QVBoxLayout()
        self._layout.addStretch(1)

        self._mainW = QWidget()
        self._mainW.setLayout(self._layout)

        self.setWidget(self._mainW)
        self.setWidgetResizable(True)

    def addWidgetItem(self, widget: QWidget):
        self._layout.insertWidget(self._numWidget, widget)
        self._numWidget += 1
    
    def delWidgetItem(self, widget: QWidget):
        self._layout.removeWidget(widget)
        self._numWidget -= 1

    def resizeEvent(self, e: QResizeEvent):
        super().resizeEvent(e)
        width = e.size().width()
        if width > 0:
            self._mainW.setMaximumWidth(width)
        



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  downloader.py
@Date    :  2021/09/15
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :  
"""
import time

from PyQt5.Qt import QThread

from tidal_gui.viewModel.taskModel import TaskModel


class DownloaderImp(QThread):
    def __init__(self):
        super(DownloaderImp, self).__init__()
        self._taskModel = None

    def run(self):
        print('DownloadImp start...')
        while not self.isInterruptionRequested():
            if self._taskModel is not None:
                item = self._taskModel.getWaitDownloadItem()
                if item is not None:
                    item.download()
            time.sleep(1)
        print('DownloadImp stop...')

    def setTaskModel(self, model: TaskModel):
        self._taskModel = model

    def stop(self):
        self.requestInterruption()
        self.wait()


downloadImp = DownloaderImp()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  tableWidget.py
@Date    :  2021/8/18
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
import threading

from PyQt5.QtCore import Qt, QUrl
from PyQt5.QtGui import QPixmap
from PyQt5.QtNetwork import QNetworkAccessManager, QNetworkRequest
from PyQt5.QtWidgets import QTableWidget, QTableWidgetItem, QAbstractItemView

from tidal_gui.control.label import Label, LabelStyle


class TableWidget(QTableWidget):
    def __init__(self, columnNames: list, rowCount: int = 20):
        super(TableWidget, self).__init__()
        self.setColumnCount(len(columnNames))
        self.setRowCount(rowCount)

        self._lock = threading.Lock()
        self._netManager = QNetworkAccessManager()

        self.columnAligns = []

        for index, name in enumerate(columnNames):
            item = QTableWidgetItem(name)

            align = Qt.AlignLeft | Qt.AlignVCenter
            if name == '#' or name == ' ':
                align = Qt.AlignCenter

            self.columnAligns.append(align)
            item.setTextAlignment(align)
            self.setHorizontalHeaderItem(index, item)

        for index in range(0, rowCount):
            self.setRowHeight(index, 50)

        self.setShowGrid(False)
        self.verticalHeader().setVisible(False)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setSelectionMode(QAbstractItemView.SingleSelection)

        self.horizontalHeader().setStretchLastSection(True)
        self.setEditTriggers(QAbstractItemView.NoEditTriggers)

        self.setFocusPolicy(Qt.NoFocus)

    def changeRowCount(self, rows: int):
        if rows != self.rowCount():
            self.setRowCount(rows)
            for index in range(0, rows):
                self.setRowHeight(index, 50)

    def addItem(self, rowIdx: int, colIdx: int, text):
        if isinstance(text, str):
            item = QTableWidgetItem(text)
            item.setTextAlignment(self.columnAligns[colIdx])
            self.setItem(rowIdx, colIdx, item)
        elif isinstance(text, QUrl):
            self.__addPicItem__(rowIdx, colIdx, text)

    def __picDownload__(self, rowIdx, colIdx):
        reply = self.sender()
        data = reply.readAll()
        if data.size() <= 0:
            return

        pic = QPixmap()
        pic.loadFromData(data)

        self._lock.acquire()
        self.cellWidget(rowIdx, colIdx).setPixmap(pic.scaled(32, 32))
        self._lock.release()

        reply.deleteLater()

    def __addPicItem__(self, rowIdx: int, colIdx: int, url: QUrl):
        icon = Label('', LabelStyle.Icon)
        icon.setAlignment(Qt.AlignCenter)

        self.setCellWidget(rowIdx, colIdx, icon)

        reply = self._netManager.get(QNetworkRequest(url))
        reply.finished.connect(lambda: self.__picDownload__(rowIdx, colIdx))

    def addWidgetItem(self, rowIdx: int, colIdx: int, widget):
        self.setCellWidget(rowIdx, colIdx, widget)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  line.py
@Date    :  2021/8/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from PyQt5.QtWidgets import QFrame


class Line(QFrame):
    def __init__(self, shape: str = 'V'):
        super(Line, self).__init__()
        self.setFrameShape(QFrame.VLine if shape == 'V' else QFrame.HLine)
        self.setFrameShadow(QFrame.Sunken)

        self.setObjectName('VLineQFrame' if shape == 'V' else 'HLineQFrame')



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  aboutModel.py
@Date    :  2021/8/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from tidal_gui.view.aboutView import AboutView
from tidal_gui.viewModel.viewModel import ViewModel


class AboutModel(ViewModel):
    def __init__(self):
        super(AboutModel, self).__init__()
        self.view = AboutView()
        self.view.setTitle('TIDAL-GUI')
        self.view.setAuthor('Yaronzz')
        self.view.setVersion('1.0.0.1')
        self.view.setLastVersion('1.0.0.1')



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  taskView.py
@Date    :  2021/8/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from enum import Enum

from PyQt5.QtCore import Qt, QSize
from PyQt5.QtWidgets import QWidget, QGridLayout, QListWidgetItem

from tidal_gui.control.label import Label
from tidal_gui.control.line import Line
from tidal_gui.control.listWidget import ListWidget
from tidal_gui.control.scrollWidget import ScrollWidget
from tidal_gui.style import LabelStyle, ListWidgetStyle
from tidal_gui.theme import getResourcePath


class TaskStatus(Enum):
    Download = 0,
    Complete = 1,
    Error = 2,


class TaskView(QWidget):
    def __init__(self):
        super(TaskView, self).__init__()
        self._listMap = {}
        self._pageMap = {}

        for item in map(lambda typeItem: typeItem.name, TaskStatus):
            self._listMap[item] = ScrollWidget()
            self._pageMap[item] = QWidget()

        self.__initView__()
        self._listTab.setCurrentRow(0)
        self._pageMap[TaskStatus.Download.name].show()

    def __initView__(self):
        grid = QGridLayout(self)
        grid.addLayout(self.__initLefTab__(), 0, 0, Qt.AlignLeft)
        for item in map(lambda typeItem: typeItem.name, TaskStatus):
            grid.addWidget(self.__createContent__(item), 0, 1)

    def __initLefTab__(self):
        self._listTab = ListWidget(ListWidgetStyle.TaskTab)
        self._listTab.setIconSize(QSize(20, 20))

        iconPath = getResourcePath() + "/svg/taskTab/"
        self._listTab.addIConTextItem(iconPath + 'download.svg', TaskStatus.Download.name)
        self._listTab.addIConTextItem(iconPath + 'complete.svg', TaskStatus.Complete.name)
        self._listTab.addIConTextItem(iconPath + 'error.svg', TaskStatus.Error.name)

        self._listTab.itemClicked.connect(self.__tabItemChanged__)

        layout = QGridLayout()
        layout.addWidget(Label("TASK LIST", LabelStyle.PageTitle), 0, 0, Qt.AlignLeft)
        layout.addWidget(self._listTab, 1, 0, Qt.AlignLeft)
        layout.addWidget(Line('V'), 0, 1, 2, 1, Qt.AlignLeft)
        return layout

    def __createContent__(self, typeStr: str):
        layout = QGridLayout()
        layout.setContentsMargins(0, 0, 10, 10)
        layout.setSpacing(10)
        layout.addWidget(Label(typeStr, LabelStyle.PageSubTitle), 0, 0, Qt.AlignTop)
        layout.addWidget(Line('H'), 1, 0, Qt.AlignTop)
        layout.addWidget(self._listMap[typeStr], 2, 0)

        self._pageMap[typeStr].setLayout(layout)
        self._pageMap[typeStr].hide()
        return self._pageMap[typeStr]

    def __tabItemChanged__(self, item: QListWidgetItem):
        for name in self._listMap:
            if name == item.text():
                self._pageMap[name].show()
            else:
                self._pageMap[name].hide()

    def addItemView(self, stype: TaskStatus, view):
        self._listMap[stype.name].addWidgetItem(view)
    
    def delItemView(self, stype: TaskStatus, view):
        self._listMap[stype.name].delWidgetItem(view)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  comboBox.py
@Date    :  2021/8/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QComboBox, QListView


class ComboBox(QComboBox):
    def __init__(self, items: list, width: int = 200):
        super(ComboBox, self).__init__()
        self.setItems(items)
        self.setFixedWidth(width)
        self.setView(QListView())
        # remove shadow
        self.view().window().setWindowFlags(Qt.Popup | Qt.FramelessWindowHint | Qt.NoDropShadowWindowHint)
        self.view().window().setAttribute(Qt.WA_TranslucentBackground)

    def setItems(self, items):
        for item in items:
            self.addItem(str(item))



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  loginModel.py
@Date    :  2021/8/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
import _thread
import webbrowser

from PyQt5.QtCore import pyqtSignal
from tidal_dl.util import API, loginByConfig, loginByWeb

from tidal_gui.view.loginView import LoginView
from tidal_gui.viewModel.viewModel import ViewModel


class LoginModel(ViewModel):
    SIGNAL_LOGIN_SUCCESS = pyqtSignal()

    def __init__(self):
        super(LoginModel, self).__init__()
        self.view = LoginView()
        self.view.connectConfirmButton(self.__openWeb__)
        self.SIGNAL_REFRESH_VIEW.connect(self.__refresh__)

    def __refresh__(self, stype: str, text: str):
        if stype == "userCode":
            self.view.setDeviceCode(text)
            self.view.enableConfirmButton(True)
            self.view.setMsg(' ')
            self.view.showEnterView()
        elif stype == "showMsg":
            self.view.hideEnterView()
            self.view.setMsg(text)

    def login(self, useConfig=True):
        self.SIGNAL_REFRESH_VIEW.emit('showMsg', "LOGIN...")

        def __thread_login__(model: LoginModel, useConfig: bool):
            if useConfig and loginByConfig():
                model.SIGNAL_LOGIN_SUCCESS.emit()
                return
            model.getDeviceCode()

        _thread.start_new_thread(__thread_login__, (self, useConfig))

    def getDeviceCode(self):
        self.SIGNAL_REFRESH_VIEW.emit('showMsg', "GET DEVICE-CODE...")

        def __thread_getCode__(model: LoginModel):
            msg, check = API.getDeviceCode()
            if check:
                model.SIGNAL_REFRESH_VIEW.emit('userCode', API.key.userCode)
            else:
                model.SIGNAL_REFRESH_VIEW.emit('showMsg', msg)

        _thread.start_new_thread(__thread_getCode__, (self,))

    def __openWeb__(self):
        self.view.enableConfirmButton(False)
        webbrowser.open('http://link.tidal.com/' + API.key.userCode, new=0, autoraise=True)

        def __thread_waitLogin__(model: LoginModel):
            if loginByWeb():
                model.SIGNAL_LOGIN_SUCCESS.emit()
            else:
                model.getDeviceCode()

        _thread.start_new_thread(__thread_waitLogin__, (self,))



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  mainModel.py
@Date    :  2021/8/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from tidal_gui.downloader import downloadImp
from tidal_gui.view.mainView import MainView
from tidal_gui.viewModel.aboutModel import AboutModel
from tidal_gui.viewModel.loginModel import LoginModel
from tidal_gui.viewModel.searchModel import SearchModel
from tidal_gui.viewModel.settingsModel import SettingsModel
from tidal_gui.viewModel.taskModel import TaskModel
from tidal_gui.viewModel.viewModel import ViewModel


class MainModel(ViewModel):
    def __init__(self):
        super(MainModel, self).__init__()
        self.loginModel = LoginModel()
        self.searchModel = SearchModel()
        self.taskModel = TaskModel()
        self.settingsModel = SettingsModel(self)
        self.aboutModel = AboutModel()

        self.loginModel.SIGNAL_LOGIN_SUCCESS.connect(self.__loginSuccess__)
        self.searchModel.SIGNAL_ADD_TASKITEM.connect(self.taskModel.addTaskItem)

        self.view = MainView()
        self.view.setSearchView(self.searchModel.view)
        self.view.setTaskView(self.taskModel.view)
        self.view.setSettingsView(self.settingsModel.view)
        self.view.setAboutView(self.aboutModel.view)

        self.view.showPage()

        downloadImp.setTaskModel(self.taskModel)
        downloadImp.start()

    def uninit(self):
        self.taskModel.uninit()
        downloadImp.stop()

    def show(self, relogin: bool = False):
        self.view.hide()
        self.loginModel.login(bool(1 - relogin))
        self.loginModel.show()

    def __loginSuccess__(self):
        self.loginModel.hide()
        self.view.show()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  taskItemModel.py
@Date    :  2021/9/14
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
import _thread
from asyncio import tasks
from enum import Enum
import os
import time

import aigpy.stringHelper

from tidal_dl import Type
from tidal_dl.model import Album, Track, Video, Playlist
from tidal_dl.util import API, getArtistsNames, getBasePath, getDurationString
from tidal_gui.view.taskItemView import TaskItemView
from tidal_gui.view.taskView import TaskStatus
from tidal_gui.viewModel.downloadItemModel import DownloadItemModel, DownloadStatus
from tidal_gui.viewModel.viewModel import ViewModel


class TaskItemModel(ViewModel):
    def __init__(self, data):
        super(TaskItemModel, self).__init__()
        self.view = TaskItemView()
        self.data = data
        self.downloadModelList = []
        self.path = ''

        if isinstance(data, Album):
            self.__initAlbum__(data)
        elif isinstance(data, Track):
            self.__initTrack__(data)
        elif isinstance(data, Video):
            self.__initVideo__(data)
        elif isinstance(data, Playlist):
            self.__initPlaylist__(data)

        self.view.connectButton('retry', self.__btnFuncRetry__)
        self.view.connectButton('cancel', self.__btnFuncCancel__)
        self.view.connectButton('delete', self.__btnFuncDelete__)
        self.view.connectButton('open', self.__btnFuncOpen__)

        self.SIGNAL_REFRESH_VIEW.connect(self.__refresh__)

    def getTaskStatus(self) -> TaskStatus:
        if len(self.downloadModelList) <= 0:
            return TaskStatus.Download
        
        errorNum = 0
        for item in self.downloadModelList:
            if item.status in [DownloadStatus.WAIT, DownloadStatus.RUNNING, DownloadStatus.CANCEL]:
                return TaskStatus.Download
            elif item.status == DownloadStatus.ERROR:
                errorNum += 1
        
        if errorNum > 0:
            return TaskStatus.Error
        return TaskStatus.Complete

    def __refresh__(self, stype: str, obj):
        if stype == "setPic":
            self.view.setPic(obj)
        elif stype == "addListItems":
            for index, item in enumerate(obj):
                downItem = DownloadItemModel(index + 1, item, self.path)
                self.view.addListItem(downItem.view)
                self.downloadModelList.append(downItem)

    def __btnFuncRetry__(self):
        for item in self.downloadModelList:
            item.retry()

    def __btnFuncCancel__(self):
        for item in self.downloadModelList:
            item.stopDownload()

    def __btnFuncDelete__(self):
        for item in self.downloadModelList:
            item.stopDownload()

    def __btnFuncOpen__(self):
        if self.path == '':
            return
        if os.path.exists(self.path):
            os.startfile(self.path)

    def __initAlbum__(self, data: Album):
        self.path = getBasePath(data)

        title = data.title
        desc = f"by {getArtistsNames(data.artists)} " \
               f"{getDurationString(data.duration)} " \
               f"Track-{data.numberOfTracks} " \
               f"Video-{data.numberOfVideos}"
        self.view.setLabel(title, desc)

        def __thread_func__(model: TaskItemModel, album: Album):
            cover = API.getCoverData(album.cover)
            model.SIGNAL_REFRESH_VIEW.emit('setPic', cover)

            msg, tracks, videos = API.getItems(album.id, Type.Album)
            if not aigpy.stringHelper.isNull(msg):
                model.view.setErrmsg(msg)
                return

            for item in tracks:
                item.album = album
            for item in videos:
                item.album = album

            model.SIGNAL_REFRESH_VIEW.emit('addListItems', tracks + videos)
            time.sleep(1)

        _thread.start_new_thread(__thread_func__, (self, data))

    def __initTrack__(self, data: Track):
        title = data.title
        desc = f"by {getArtistsNames(data.artists)} " \
               f"{getDurationString(data.duration)} "
        self.view.setLabel(title, desc)

        def __thread_func__(model: TaskItemModel, track: Track):
            mag, track.album = API.getAlbum(track.album.id)
            model.path = getBasePath(track)
            cover = API.getCoverData(track.album.cover)
            model.SIGNAL_REFRESH_VIEW.emit('setPic', cover)
            model.SIGNAL_REFRESH_VIEW.emit('addListItems', [track])
            time.sleep(1)

        _thread.start_new_thread(__thread_func__, (self, data))

    def __initVideo__(self, data: Video):
        self.path = getBasePath(data)

        title = data.title
        desc = f"by {getArtistsNames(data.artists)} " \
               f"{getDurationString(data.duration)} "
        self.view.setLabel(title, desc)

        def __thread_func__(model: TaskItemModel, video: Video):
            cover = API.getCoverData(video.imageID)
            model.SIGNAL_REFRESH_VIEW.emit('setPic', cover)
            model.SIGNAL_REFRESH_VIEW.emit('addListItems',  [video])
            time.sleep(1)

        _thread.start_new_thread(__thread_func__, (self, data))

    def __initPlaylist__(self, data: Playlist):
        self.path = getBasePath(data)

        title = data.title
        desc = f"{getDurationString(data.duration)} " \
               f"Track-{data.numberOfTracks} " \
               f"Video-{data.numberOfVideos}"
        self.view.setLabel(title, desc)

        def __thread_func__(model: TaskItemModel, playlist: Playlist):
            cover = API.getCoverData(playlist.squareImage)
            model.SIGNAL_REFRESH_VIEW.emit('setPic', cover)

            msg, tracks, videos = API.getItems(playlist.uuid, Type.Playlist)
            if not aigpy.stringHelper.isNull(msg):
                model.view.setErrmsg(msg)
                return

            for item in tracks:
                mag, album = API.getAlbum(item.album.id)
                item.playlist = playlist
                item.album = album
            for item in videos:
                item.playlist = playlist

            model.SIGNAL_REFRESH_VIEW.emit('addListItems', tracks + videos)
            time.sleep(1)

        _thread.start_new_thread(__thread_func__, (self, data))



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  layout.py
@Date    :  2021/8/13
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from PyQt5.QtWidgets import QHBoxLayout, QVBoxLayout


def createHBoxLayout(widgets):
    layout = QHBoxLayout()
    for item in widgets:
        layout.addWidget(item)
    return layout


def createVBoxLayout(widgets):
    layout = QVBoxLayout()
    for item in widgets:
        layout.addWidget(item)
    return layout



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  __init__.py
@Date    :  2021/05/08
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
import sys

from PyQt5.QtWidgets import QApplication

from tidal_gui import theme
from tidal_gui.viewModel.mainModel import MainModel


def main():
    qss = theme.getThemeQssContent()
    app = QApplication(sys.argv)
    app.setStyleSheet(qss)

    mainView = MainModel()
    mainView.show()

    app.exec_()
    mainView.uninit()

    sys.exit()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  listWidget.py
@Date    :  2021/8/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QListWidget, QListWidgetItem, QWidget

from tidal_gui.style import ListWidgetStyle


class ListWidget(QListWidget):
    def __init__(self, style: ListWidgetStyle = ListWidgetStyle.Default):
        super(ListWidget, self).__init__()
        self.setObjectName(style.name + "ListWidget")

    def addIConTextItem(self, iconUrl: str, text: str):
        self.addItem(QListWidgetItem(QIcon(iconUrl), text))

    def addWidgetItem(self, widget: QWidget):
        item = QListWidgetItem(self)
        # item.setSizeHint(QSize(widget.width(), widget.height()))
        self.addItem(item)
        self.setItemWidget(item, widget)

    def setAdjustMode(self):
        self.setResizeMode(QListWidget.Adjust)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  searchView.py
@Date    :  2021/8/17
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
import threading

from PyQt5.QtCore import Qt, QUrl
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QTabWidget

import tidal_dl.model
from tidal_dl import Type
from tidal_dl.util import API, getDurationString
from tidal_gui.control.comboBox import ComboBox
from tidal_gui.control.label import Label
from tidal_gui.control.lineEdit import LineEdit
from tidal_gui.control.pushButton import PushButton
from tidal_gui.control.tableWidget import TableWidget
from tidal_gui.style import ButtonStyle, LabelStyle
from tidal_gui.theme import getResourcePath


class SearchView(QWidget):
    def __init__(self):
        super(SearchView, self).__init__()
        self._rowCount = 20
        self._table = {}
        self._lock = threading.Lock()
        self.__initView__()
        self._searchEdit.setFocus()

    def __initView__(self):
        grid = QVBoxLayout(self)
        grid.addLayout(self.__initHead__(), Qt.AlignTop)
        grid.addLayout(self.__initContent__())
        grid.addLayout(self.__initTail__(), Qt.AlignBottom)

    def __initHead__(self):
        self._searchEdit = LineEdit(iconUrl=getResourcePath() + "/svg/search2.svg")
        self._searchErrLabel = Label('', LabelStyle.SearchErr)
        self._searchErrLabel.hide()
        layout = QVBoxLayout()
        layout.addWidget(self._searchEdit)
        layout.addWidget(self._searchErrLabel)
        layout.setContentsMargins(0, 0, 0, 5)
        return layout

    def __initTail__(self):
        self._trackQualityComboBox = ComboBox([], 150)
        self._videoQualityComboBox = ComboBox([], 150)

        self._prePageBtn = PushButton('', ButtonStyle.PrePage)
        self._nextPageBtn = PushButton('', ButtonStyle.NextPage)

        self._pageIndexEdit = LineEdit('')
        self._pageIndexEdit.setAlignment(Qt.AlignCenter)
        self._pageIndexEdit.setEnabled(False)

        self._downloadBtn = PushButton('Download', ButtonStyle.Primary)

        layout = QHBoxLayout()
        layout.addWidget(Label('Track:'))
        layout.addWidget(self._trackQualityComboBox)
        layout.addWidget(Label('Video:'))
        layout.addWidget(self._videoQualityComboBox)
        layout.addStretch(1)
        layout.addWidget(self._prePageBtn)
        layout.addWidget(self._pageIndexEdit)
        layout.addWidget(self._nextPageBtn)
        layout.addWidget(self._downloadBtn)
        return layout

    def __initContent__(self):
        self._tabWidget = QTabWidget(self)
        self._tabWidget.addTab(self.__initTable__(Type.Album), "ALBUM")
        self._tabWidget.addTab(self.__initTable__(Type.Track), "TRACK")
        self._tabWidget.addTab(self.__initTable__(Type.Video), "VIDEO")
        self._tabWidget.addTab(self.__initTable__(Type.Playlist), "PLAYLIST")

        layout = QVBoxLayout()
        layout.addWidget(self._tabWidget)
        return layout

    def __initTable__(self, stype: Type):
        columnHeads = []
        columnWidths = []
        if stype == Type.Album:
            columnHeads = ['#', ' ', ' ', 'Title', 'Artists', 'Release', 'Duration']
            columnWidths = [50, 60, 60, 400, 200, 200]
        elif stype == Type.Track:
            columnHeads = ['#', ' ', 'Title', 'Album', 'Artists', 'Duration']
            columnWidths = [50, 60, 400, 200, 200]
        elif stype == Type.Video:
            columnHeads = ['#', ' ', ' ', 'Title', 'Artists', 'Duration']
            columnWidths = [50, 60, 60, 400, 200, 200]
        elif stype == Type.Playlist:
            columnHeads = ['#', ' ', 'Title', 'Artist', 'Duration']
            columnWidths = [50, 60, 400, 200, 200]

        self._table[stype] = TableWidget(columnHeads, self._rowCount)
        for index, width in enumerate(columnWidths):
            self._table[stype].setColumnWidth(index, width)

        return self._table[stype]

    def __clearTableRowItems__(self, stype: Type, fromIndex: int):
        endIndex = self._rowCount - 1
        columnNum = self._table[stype].columnCount()
        while fromIndex <= endIndex:
            for colIdx in range(0, columnNum):
                self._table[stype].addItem(fromIndex, colIdx, '')
            fromIndex += 1

    def setTableItems(self, stype: Type, indexOffset: int, result: tidal_dl.model.SearchResult):
        if stype == Type.Album:
            items = result.albums.items
            datas = []
            for index, item in enumerate(items):
                rowData = [str(index + 1 + indexOffset),
                           QUrl(API.getCoverUrl(item.cover)),
                           API.getFlag(item, Type.Album, True),
                           item.title,
                           item.artists[0].name,
                           str(item.releaseDate),
                           getDurationString(item.duration)]
                datas.append(rowData)

        elif stype == Type.Track:
            items = result.tracks.items
            datas = []
            for index, item in enumerate(items):
                rowData = [str(index + 1 + indexOffset),
                           API.getFlag(item, Type.Track, True),
                           item.title,
                           item.album.title,
                           item.artists[0].name,
                           getDurationString(item.duration)]
                datas.append(rowData)

        elif stype == Type.Video:
            items = result.videos.items
            datas = []
            for index, item in enumerate(items):
                rowData = [str(index + 1 + indexOffset),
                           QUrl(API.getCoverUrl(item.imageID)),
                           API.getFlag(item, Type.Video, True),
                           item.title,
                           item.artists[0].name,
                           getDurationString(item.duration)]
                datas.append(rowData)

        elif stype == Type.Playlist:
            items = result.playlists.items
            datas = []
            for index, item in enumerate(items):
                rowData = [str(index + 1 + indexOffset),
                           QUrl(API.getCoverUrl(item.squareImage)),
                           item.title,
                           '',
                           getDurationString(item.duration)]
                datas.append(rowData)

        for index, rowData in enumerate(datas):
            for colIdx, obj in enumerate(rowData):
                self._table[stype].addItem(index, colIdx, obj)

        self.__clearTableRowItems__(stype, len(items))
        self._table[stype].viewport().update()

    def getSearchText(self):
        return self._searchEdit.text()

    def setSearchErrmsg(self, text: str):
        self._searchErrLabel.setText(text)
        if text != '':
            self._searchErrLabel.show()
        else:
            self._searchErrLabel.hide()

    def setPageIndex(self, index, sum):
        self._pageIndexEdit.setText(str(index) + '/' + str(sum))

    def getPageIndex(self) -> (int, int):
        nums = self._pageIndexEdit.text().split('/')
        return int(nums[0]), int(nums[1])

    def getSelectedTabIndex(self):
        return self._tabWidget.currentIndex()

    def getSelectedTableIndex(self, stype: Type):
        array = self._table[stype].selectedIndexes()
        if len(array) <= 0:
            return -1
        return array[0].row()

    def setTrackQualityItems(self, items: list, curItem = None):
        self._trackQualityComboBox.setItems(items)
        if curItem is not None:
            self._trackQualityComboBox.setCurrentText(curItem)

    def setVideoQualityItems(self, items: list, curItem=None):
        self._videoQualityComboBox.setItems(items)
        if curItem is not None:
            self._videoQualityComboBox.setCurrentText(curItem)

    def getTrackQualityText(self):
        return self._trackQualityComboBox.currentText()

    def getVideoQualityText(self):
        return self._videoQualityComboBox.currentText()

    def connectButton(self, name: str, func):
        if name == 'search':
            self._searchEdit.returnPressed.connect(func)
        elif name == 'prePage':
            self._prePageBtn.clicked.connect(func)
        elif name == 'nextPage':
            self._nextPageBtn.clicked.connect(func)
        elif name == 'download':
            self._downloadBtn.clicked.connect(func)

    def connectTab(self, func):
        self._tabWidget.currentChanged.connect(func)
    
    def connectQualityComboBox(self, name: str, func):
        if name == 'track':
            self._trackQualityComboBox.currentIndexChanged.connect(func)
        else:
            self._videoQualityComboBox.currentIndexChanged.connect(func)
